@@ -27,16 +27,18 @@
 #include "shrpx_memcached_request.h"
 #include "shrpx_memcached_connection.h"
 #include "shrpx_config.h"
+#include "shrpx_log.h"
 
 namespace shrpx {
 
 MemcachedDispatcher::MemcachedDispatcher(const Address *addr,
                                          struct ev_loop *loop, SSL_CTX *ssl_ctx,
                                          const StringRef &sni_name,
-                                         MemchunkPool *mcpool)
+                                         MemchunkPool *mcpool,
+                                         std::mt19937 &gen)
     : loop_(loop),
       mconn_(make_unique<MemcachedConnection>(addr, loop_, ssl_ctx, sni_name,
-                                              mcpool)) {}
+                                              mcpool, gen)) {}
 
 MemcachedDispatcher::~MemcachedDispatcher() {}
 
