@@ -25,6 +25,10 @@
 #ifndef NGHTTP2_HELPER_TEST_H
 #define NGHTTP2_HELPER_TEST_H
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif /* HAVE_CONFIG_H */
+
 void test_nghttp2_adjust_local_window_size(void);
 void test_nghttp2_check_header_name(void);
 void test_nghttp2_check_header_value(void);
